/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import ru.ispras.fortress.util.InvariantChecks;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public final class DataGeneratorRegistry {
  private final Map<String, List<DataGenerator>> registry = new HashMap<>();

  public DataGeneratorRegistry() {
    // Universal data generator
    add(GeneratorZero.NAME, new GeneratorZero());
    add(GeneratorRandom.NAME, new GeneratorRandom());
    add(GeneratorRandomBiased.NAME, new GeneratorRandomBiased());

    // Integer data generators. Have limitations. Tied to argument names.
    add(GeneratorIntAdd.NAME, new GeneratorIntAdd());
    add(GeneratorIntSub.NAME, new GeneratorIntSub());

    // Floating-point data generators. Have limitations. Tied to argument names.
    add(GeneratorFpAdd.NAME, new GeneratorFpAdd());
    add(GeneratorFpSub.NAME, new GeneratorFpSub());

    // Just to show how to create data generators for constraints.
    add(GeneratorConstraintIntOverflow.NAME, new GeneratorConstraintIntOverflow());

    // Boundary data generator.
    add(GeneratorBoundaryValues.NAME, new GeneratorBoundaryValues());

    // Range data generator.
    add(GeneratorRange.NAME, new GeneratorRange());
  }

  public boolean add(final String name, final DataGenerator generator) {
    InvariantChecks.checkNotNull(name);
    InvariantChecks.checkNotNull(generator);

    List<DataGenerator> list = registry.get(name);
    if (list == null) {
      list = new ArrayList<>();
      list.add(generator);
      registry.put(name, list);
    } else {
      for (final DataGenerator stored : list) {
        if (stored.equals(generator)) {
          return false;
        }
      }
      list.add(generator);
    }
    return true;
  }

  public List<DataGenerator> getNamed(final String name) {
    InvariantChecks.checkNotNull(name);

    final List<DataGenerator> generators = registry.get(name);
    if (generators != null) {
      return Collections.unmodifiableList(generators);
    }

    return Collections.emptyList();
  }
}
