/*
 * Copyright 2018 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import ru.ispras.fortress.data.DataTypeId;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.testbase.knowledge.iterator.IntRangeIterator;
import ru.ispras.testbase.knowledge.iterator.Iterator;

/**
 * The {@link GeneratorRange} class generates a series of values that follow
 * the specified integer range.
 *
 * @author <a href="mailto:andrewt@ispras.ru">Andrei Tatarnikov</a>
 */
final class GeneratorRange extends DataGeneratorBase {
  public static final String NAME = "range";

  public GeneratorRange() {
    super(NAME);
  }

  @Override
  protected void generate(final DataGeneratorQuery query) {
    final int min = query.getIntegerParameter("min").intValue();
    final int max = query.getIntegerParameter("max").intValue();

    final int inc = query.hasParameter("inc") ?
        query.getIntegerParameter("inc").intValue() : 1;

    for (final DataGeneratorVariable variable : query.getUnknownVariables()) {
      if (variable.isType(DataTypeId.BIT_VECTOR)) {
        setValue(variable, newRangeIterator(variable.getSize(), min, max, inc));
      } else {
        throw new IllegalArgumentException(String.format(
            "Variable %s has unsupported type %s.", variable.getName(), variable.getType()));
      }
    }
  }

  private static Iterator<NodeValue> newRangeIterator(
      final int valueSize,
      final int min,
      final int max,
      final int inc) {
    final IntRangeIterator intRangeIterator = new IntRangeIterator(min, max, inc);
    return new IntegerToNodeValueIterator(valueSize, intRangeIterator);
  }

  private static class IntegerToNodeValueIterator implements Iterator<NodeValue> {
    private final int valueSize;
    private final Iterator<Integer> integerIterator;

    public IntegerToNodeValueIterator(final int valueSize, final Iterator<Integer> integerIterator) {
      this.valueSize = valueSize;
      this.integerIterator = integerIterator;
    }

    @Override
    public void init() {
      integerIterator.init();
    }

    @Override
    public boolean hasValue() {
      return integerIterator.hasValue();
    }

    @Override
    public NodeValue value() {
      return NodeValue.newBitVector(integerIterator.value(), valueSize);
    }

    @Override
    public void next() {
      integerIterator.next();
    }

    @Override
    public void stop() {
      integerIterator.stop();
    }

    @Override
    public Iterator<NodeValue> clone() {
      throw new UnsupportedOperationException();
    }
  }
}
